(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This script contains definitions of different utility classes used by other components.

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also uses these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//***********************************
// MultiSystem V1.1
//
// FX - definitions of different utility classes used by other components
//      of MultiSystem V1.1
//
// Adrian Michel - 2003
//***********************************


//***********************************
// Assert procedure
//
// used for debug purposes, when there are assumptions to be checked
// such as in case of procedures and function input parameters or
// other when a certain condition is considered a programming error
// rather than a runtime normal exceptional condition
//
// used only scarcely in this version of the script for performance reasons
//***********************************
 // assertOn used for performance reasons - if disabled, it is faster to test a variable than to make a call into assert
var assertOn : boolean;
// set on in debug mode
assertOn := true;

procedure assert( value : boolean; message : String );
begin
  if( not value ) then
  begin
    var msg : String;
    msg := 'Assert failed: ' + message + '. Script will exit';
    ShowMessage( msg );
    print( msg );
    Abort;
  end;
end;

//***********************************
// Settable value class
//
// base class for classes that need to indicate that they have been 'set'
//***********************************
type Settable = class
  private
    _set : boolean;

  protected
    constructor Create;
    procedure s;
  public function isSet : boolean;
end;

constructor Settable.Create;
begin
  _set := false;
end;

procedure Settable.s;
begin
  _set := true;
end;

function Settable.isSet : boolean;
begin
  Result := _set;
end;

//***********************************
// SettableFloat value class
//
// a class containing one settable Float value
//***********************************
type SettableFloat = class( Settable )
private _value : float;

public constructor Create;
public procedure set( v : float );
public function get : float;
end;

constructor SettableFloat.Create;
begin
  inherited Create;
end;

procedure SettableFloat.set( v : float );
begin
  _value := v;
  s;
end;

function SettableFloat.get : float;
begin
  Result := _value;
end;

//***********************************
// Settable2Float class
//
// class containing two settable float values.
//***********************************
type Settable2Float = class( Settable )
private _value1 : float;
private _value2 : float;
public constructor Create;
public procedure set( value1, value2 : float );
public function get1 : float;
public function get2 : float;
end;

constructor Settable2Float.Create;
begin
  inherited Create;
end;

procedure Settable2Float.set( value1, value2 : float );
begin
  _value1 := value1;
  _value2 := value2;
  s;
end;

function Settable2Float.get1 : float;
begin
  Result := _value1;
end;

function Settable2Float.get2 : float;
begin
  Result := _value2;
end;

//***********************************
// SettableInt value class
//
// class containing one settable int
//***********************************
type SettableInt = class( Settable )
private _value : integer;
public constructor Create;
public procedure set( value : integer );
public function get : integer;
end;

constructor SettableInt.Create;
begin
  inherited Create;
end;

procedure SettableInt.set( value : integer );
begin
  _value := value;
  s;
end;

function SettableInt.get : integer;
begin
  Result := _value;
end;

//***********************************
// PositionInfo class
//
// stores info requied on a per position basis, interbar: break even and trailing stop
// break even and trailing stop get triggered on one bar but exit the position on
// a different bar, so need to store these values for later
//***********************************
type PositionInfo = class
  private _breakEvenActive : boolean;
  private _breakEvenDirection : boolean;

  private _trailingStopActive : boolean;
  private _lastTrailingStopLevel : float;

  public
    constructor Create;
    procedure activateBreakEvenStop( direction : boolean );
    function isBreakEvenStopActive : boolean;
    function breakEvenStopDirection : boolean;

    procedure activateTrailingStop( level : float );
    function isTrailingStopActivated : boolean;
    function getTrailingStopLevel : float;
end;

procedure PositionInfo.activateTrailingStop( level : float );
begin
  _trailingStopActive := true;
  _lastTrailingStopLevel := level;
end;

function PositionInfo.isTrailingStopActivated : boolean;
begin
  Result := _trailingStopActive;
end;

function PositionInfo.getTrailingStopLevel : float;
begin
  Result := _lastTrailingStopLevel;
end;

constructor PositionInfo.Create;
begin
  _breakEvenActive := false;
  _trailingStopActive := false;
end;

procedure PositionInfo.activateBreakEvenStop( direction : boolean );
begin
  _breakEvenActive := true;
  _breakEvenDirection := direction;
end;

function PositionInfo.isBreakEvenStopActive : boolean;
begin
  Result := _breakEvenActive;
end;

function PositionInfo.breakEvenStopDirection : boolean;
begin
  Result := _breakEvenDirection;
end;

//***********************************
// ActivePos class
//
// contains the a list/array of indexes pointing to currently active positions
// on a per system basis
// NOTE: multiple levels of indirections are used in order to achieve
// the virtualization of the position context for each system, so these
// indexes point to elements found in an array of all positions, active or closed, also
// per system, and each of the elements in this second array points to the actual position
//***********************************
type
  ActivePos = class

  private
  _pos : TList;

  public
  constructor Create;
  procedure Free;

   // TODO: check for errors, what if the position is not in there?
   // TODO: implement an assert class
  procedure add( pos : integer );
  procedure remove( pos : integer );

  function count : integer;
  function last : integer;
  function get( index : integer ) : integer;
  procedure activateBreakEvenStop( index : integer; direction : boolean );
  function isBreakEvenStopActive( index : integer ) : boolean;
  function breakEvenStopDirection( index : integer ) : boolean;

  function isTrailingStopActivated( index : integer ) : boolean;
  function getTrailingStopLevel( index : integer ) : float;
  procedure setTrailingStopLevel( index : integer; level : float );
end;

procedure ActivePos.activateBreakEvenStop( index : integer; direction : boolean );
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  o.activateBreakEvenStop( direction );
end;

function ActivePos.isBreakEvenStopActive( index : integer ) : boolean;
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  Result := o.isBreakEvenStopActive;
end;

function ActivePos.isTrailingStopActivated( index : integer ) : boolean;
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  Result := o.isTrailingStopActivated;
end;

function ActivePos.getTrailingStopLevel( index : integer ) : float;
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  Result := o.getTrailingStopLevel;
end;

procedure ActivePos.setTrailingStopLevel( index : integer; level : float );
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  o.activateTrailingStop( level );
end;

function ActivePos.breakEvenStopDirection( index : integer ) : boolean;
begin
  var o : PositionInfo;

  o := _pos.Object( index ) as PositionInfo;
  Result := o.breakEvenStopDirection;
end;

constructor ActivePos.Create;
begin
  _pos := TList.Create;
end;

procedure ActivePos.Free;
begin
  var n : integer;
  for n := 0 to _pos.count - 1 do
  begin
    var o : PositionInfo;

    o := _pos.Object( n ) as PositionInfo;
  end;
end;

//TODO: check for duplicate entries (is pos already there?)
procedure ActivePos.add( pos : integer );
begin
  _pos.AddObject( pos, PositionInfo.Create );
end;

//TODO: check for removing a non existing index
procedure ActivePos.remove( pos : integer );
begin
  if( pos >= 0 ) then
  begin
    var ct : integer;
    var o : PositionInfo;
    for ct := 0 to _pos.Count - 1 do
    begin
      if _pos.Item( ct ) = pos then
      begin
        o := _pos.Object( ct ) as PositionInfo;
       _pos.Delete( ct );
       break;
      end
    end
  end;
end;

function ActivePos.count : integer;
begin
  Result := _pos.Count;
end;

function ActivePos.last : integer;
begin
  if( _pos.count = 0 ) then
    Result := -1
  else
    Result := _pos.Item( _pos.Count - 1 );
end;

//TODO: check that index is not out of bounds
function ActivePos.get( index : integer ) : integer;
begin
  Result := _pos.Item( index );
end;

//***********************************
// Position size context
//
// this class contains per system position data, and it is called to set
// the global value before each bar iteration, thus creating a virtual
// per system position setting
//
// Note: SetPositionSizeFixed, Pct, Shares only work in SimuScript
// Only SetPositionSize works in scripts themselves. For the moment, this is not
// used though - do not see a big benefit and there may be some performance penalty
// because of all context switches before each bar.
// Another way of implementing the context switch would be to run the systems
// sequentially and do one context switch before each system, which would have a
// much more limited performance impact.
//***********************************

const FIXED_SIZE = 0;
const PCT_SIZE = 1;
const SHARES_SIZE = 2;
const SIZE_SIZE = 3;

//TODO: the default this is set by default to be 5000. It should normally
// get its value from the settings but there is no GetPositionSize function
// available, so this may lead to the settings not working.
const DEFAULT_POSITION_SIZE = 5000;
type PositionSizeContext = class
  private

  private _sizeType : integer;

  private _floatSize : float;
  private _intSize : integer;

  public constructor Create;
  public procedure fixed( size : float );
  public procedure pct( size : float );
  public procedure shares( size : integer );
  procedure switch;

  class procedure posSizeSet;
  class function isPosSizeSet : boolean;
end;

// simulate a static data member to indicate the fact that
// a position size function was called
// this will be used for performance reasons, so we won't make
// context switches if none of the position size methods was called
var _posSizeSet : boolean;
_posSizeSet := false;

class procedure PositionSizeContext.posSizeSet;
begin
  _posSizeSet := true;
end;

class function PositionSizeContext.isPosSizeSet : boolean;
begin
  Result := _posSizeSet;
end;

constructor PositionSizeContext.Create;
begin
  _sizeType := FIXED_SIZE;
  _floatSize := DEFAULT_POSITION_SIZE;
end;

procedure PositionSizeContext.fixed( size : float );
begin
  _sizeType := FIXED_SIZE;
  _floatSize := size;
  posSizeSet;
end;

procedure PositionSizeContext.pct( size : float );
begin
  _sizeType := PCT_SIZE;
  _floatSize := size;
  posSizeSet;
end;

procedure PositionSizeContext.shares( size : integer );
begin
  _sizeType := SHARES_SIZE;
  _intSize := size;
  posSizeSet;
end;

procedure PositionSizeContext.switch;
begin
{  if( isPosSizeSet ) then
  begin
    case _sizeType of
      FIXED_SIZE:
         begin
         print( 'setPositionSize: ' + FloatToStr( _floatSize ) );
         SetPositionSizeFixedX( _floatSize );
         end;
      PCT_SIZE:
         SetPositionSizePctX( _floatSize );
      SHARES_SIZE:
         SetPositionSizeSharesX( _intSize );
    else
      assert( false, 'not a valid position size type' );
    end;
  end;
}end;

//*****************************************
// String extension class
// controls how user defined strings are appended/prepended
// to other strings. Used by the class system to
// control the per system trade names.
//*****************************************
type StringExtension = class
  private
    _extensionType : integer;
    _str : string;

  public
    constructor Create;
    procedure append( str : string );
    procedure prepend( str : string );
    procedure reset;
    function apply( str : string ) : string;
end;

const APPEND_TYPE = 0;
const PREPEND_TYPE = 1;
const NO_TYPE = 2;
const SEPARATOR = ' : ';

constructor StringExtension.Create;
begin
  reset;
end;

procedure StringExtension.append( str : string );
begin
  if Length( str ) = 0 then
    _extensionType := NO_TYPE
  else
  begin
    _extensionType := APPEND_TYPE;
    _str := str;
  end;
end;

procedure StringExtension.prepend( str : string );
begin
  _extensionType := PREPEND_TYPE;
  _str := str;
end;

procedure StringExtension.reset;
begin
  _extensionType := NO_TYPE;
end;

function StringExtension.apply( str : string ) : string;
begin
  var sep : string;
  if( Length( str ) = 0 ) then
    sep := ''
  else
    sep := SEPARATOR;

  case _extensionType of
    APPEND_TYPE:
      Result := str + sep + _str;
    PREPEND_TYPE:
      Result := _str + sep + str;
    NO_TYPE:
      Result := str;
    else
      assert( false, 'StringExtension invalid extension type' );
  end;
end;

//****************************
//
// MultiSystem V1.1
// GX - functions and procedure that make the connection between
//      the WLScript API calls in a MultiSystem and the native APIs
//      with the same name
//
// Adrian Michel 2003
//****************************

//****************************
// X functions and procedures
//
// used for redirecting the calls from the class Positions
// to the right WealthScript functions/procedures
// This extra level of indirection needed because of the name
// resultions scheme which does not allow for redefintion of
// the WL names in a class and then calling the global version
// from methods within the same class
// ***************************

function GetPositionDataX( Position : integer ) : float;
begin
  Result := GetPositionData( Position );
end;

function GetPositionRiskStopX( Position : integer ) : float;
begin
  Result := GetPositionRiskStop( Position );
end;

function PositionLongX( Position: integer ): boolean;
begin
  Result := PositionLong( Position );
end;

function PositionActiveX( Position : integer ) : boolean;
begin
  Result := PositionActive( Position );
end;

function PositionEntryBarX( Position : integer ) : integer;
begin
  Result := PositionEntryBar( Position );
end;

function PositionEntryPriceX( Position: integer ): float;
begin
  Result := PositionEntryPrice( Position );
end;

function PositionExitBarX( Position: integer ): integer;
begin
  Result := PositionExitBar( Position );
end;

function PositionExitPriceX( Position: integer ): float;
begin
  Result := PositionExitPrice( Position );
end;

function PositionMAEPctX( Position: integer ): float;
begin
  Result := PositionMAEPct( Position );
end;

function PositionMFEX( Position: integer ): float;
begin
  Result := PositionMFE( Position );
end;

function PositionMFEPctX( Position: integer ): float;
begin
  Result := PositionMFEPct( Position );
end;

function PositionMAEX( Position: integer ): float;
begin
  Result := PositionMAE( Position );
end;

function PositionOpenMAEX( Position, Bar: integer ): float;
begin
  Result := PositionOpenMAE( Position , Bar );
end;

function PositionOpenMAEPctX( Position, Bar: integer ): float;
begin
  Result := PositionOpenMAEPct( Position, Bar );
end;

function PositionOpenMFEX( Position, Bar: integer ): float;
begin
  Result := PositionOpenMFE( Position, Bar );
end;

function PositionOpenMFEPctX( Position, Bar: integer ): float;
begin
  Result := PositionOpenMFEPct( Position, Bar );
end;

function PositionOpenProfitX( Bar, Position: integer ): float;
begin
  Result := PositionOpenProfit( Bar, Position );
end;

function PositionOpenProfitPctX( Bar, Position: integer ): float;
begin
  Result := PositionOpenProfitPct( Bar, Position );
end;

function PositionProfitX( Position: integer ): float;
begin
  Result := PositionProfit( Position );
end;

function PositionProfitPctX( Position: integer ): float;
begin
  Result := PositionProfitPct( Position );
end;

function PositionSharesX( Position: integer ): integer;
begin
  Result := PositionShares( Position );
end;

function PositionShortX( Position: integer ): boolean;
begin
  Result := PositionShort( Position );
end;

function PositionSignalNameX( Position: integer ): string;
begin
  Result := PositionSignalName( Position );
end;

function PositionSymbolX( Position : integer ): string;
begin
  Result := PositionSymbol( Position );
end;

procedure SetPositionDataX( Position: integer; Value: float );
begin
  SetPositionData( Position, Value );
end;

procedure SetPositionRiskStopX( Position: integer; StopLevel: float );
begin
  SetPositionRiskStop( Position, StopLevel );
end;

function PositionCountX : integer;
begin
  Result := PositionCount;
end;

//**************************************
// Buy/Short/Sell/Cover methods
//**************************************
function BuyAtCloseX( Bar : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  BuyAtClose( Bar, SignalName );
  Result := ActivePositionCount <> count;
end;

function BuyAtLimitX( Bar : integer; LimitPrice : float; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := BuyAtLimit( Bar, LimitPrice, SignalName ) and ( ActivePositionCount <> count );
end;

function BuyAtMarketX( Bar : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  BuyAtMarket( Bar, SignalName );
  Result := ActivePositionCount <> count;
end;

function BuyAtStopX( Bar : integer; StopPrice : float; SignalName : string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := BuyAtStop( Bar, StopPrice, SignalName ) and ( ActivePositionCount <> count );
end;

function CoverAtCloseX( Bar, Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  CoverAtClose( Bar, Position, SignalName );
  Result := ActivePositionCount <> count;
end;

function CoverAtLimitX( Bar : integer; LimitPrice : float; Position: integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := CoverAtLimit( Bar, LimitPrice, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function CoverAtMarketX( Bar, Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  CoverAtMarket( Bar, Position, SignalName );
  Result := ActivePositionCount <> count;
end;

function CoverAtStopX( Bar : integer; StopPrice: float; Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := CoverAtStop( Bar, StopPrice, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function SellAtCloseX( Bar, Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  SellAtClose( Bar, Position, SignalName );
  Result := ActivePositionCount <> count;
end;

function SellAtLimitX( Bar : integer; LimitPrice : float; Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := SellAtLimit( Bar, LimitPrice, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function SellAtMarketX( Bar, Position : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  SellAtMarket( Bar, Position, SignalName );
  Result := ActivePositionCount <> count;
end;

function SellAtStopX( Bar: integer; StopPrice: float; Position: integer; SignalName: string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := SellAtStop( Bar, StopPrice, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function ShortAtCloseX( Bar : integer; SignalName : string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  ShortAtClose( Bar, SignalName );
  Result := ActivePositionCount <> count;
end;

function ShortAtLimitX( Bar: integer; LimitPrice: float; SignalName: string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := ShortAtLimit( Bar, LimitPrice, SignalName ) and ( ActivePositionCount <> count );
end;

function ShortAtMarketX( Bar: integer; SignalName: string ) : boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  ShortAtMarket( Bar, SignalName );
  Result := ActivePositionCount <> count;
end;

function ShortAtStopX( Bar: integer; StopPrice: float; SignalName: string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := ShortAtStop( Bar, StopPrice, SignalName ) and ( ActivePositionCount <> count );
end;

function SellAtTrailingStopX( Bar: integer; Price: float; Position: integer; SignalName: string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := SellAtTrailingStop( Bar, Price, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function CoverAtTrailingStopX( Bar: integer; Price: float; Position: integer; SignalName: string ): boolean;
begin
  var count : integer;
  count := ActivePositionCount;
  Result := CoverAtTrailingStop( Bar, Price, Position, SignalName ) and ( ActivePositionCount <> count );
end;

function SplitPositionX( Position: integer; RetainPct: float ): integer;
begin
  Result := splitPosition( Position, RetainPct );
end;

//***************************************
function PositionBasisPriceX( Position: integer ): float;
begin
  Result := PositionBasisPrice( Position );
end;

function LastPositionX : integer;
begin
  Result := LastPosition;
end;
procedure SetPositionSizeX( Size : float );
begin
  SetPositionSize( Size );
end;

procedure SetPositionSizeFixedX( Value : float );
begin
  SetPositionSizeFixed( Value );
end;

procedure SetPositionSizePctX( Value : float );
begin
  SetPositionSizePct( Value );
end;

procedure SetPositionSizeSharesX( Value : integer );
begin
  SetPositionSizeShares( Value );
end;

//******************************************
// AutoStop methods
procedure InstallProfitTargetX( TargetLevel: float );
begin
  InstallProfitTarget( TargetLevel );
end;

procedure InstallBreakEvenStopX( Trigger: float );
begin
  InstallBreakEvenStop( Trigger );
end;

procedure InstallReverseBreakEvenStopX( LossLevel: float );
begin
  InstallReverseBreakEvenStop( LossLevel );
end;

procedure InstallStopLossX( StopLevel: float );
begin
  InstallStopLoss( StopLevel );
end;

procedure InstallTimeBasedExitX( Bars: integer );
begin
  InstallTimeBasedExit( Bars );
end;

procedure InstallTrailingStopX( Trigger, StopLevel: float );
begin
  InstallTrailingStop( Trigger, StopLevel );
end;

procedure ApplyAutoStopsX( Bar: integer );
begin
  ApplyAutoStops( Bar );
end;
