(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This is a component of the sample code that implements a filtered system i.e. with price and volume filters as well as a symbol filter used to run different component systems on different sets of symbols.

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also utilizes these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//*****************************************
//
// MultiSystem V1.1
//
// FilteredSystem class - shows how to override various filters defined in the System or MultiSystem classes
//         there are buy and short filters that only allow stocks with a certain volume average and price
//         symbol filter only enables the system when the current symbol is in the symbol list for this system on disk
//
//         in order to use the symbol filtering, use the second constructor CreateWithSymbols
//
// Adrian Michel 2003
// //*****************************************

{$I 'SymbolsList'}

type FilteredSystem = class( System )
     private _avgVol : integer;
     private _volatility : integer;
     private _symbolsList : SymbolsList;
     private _hasSymbolsList : boolean;

     // constructor
     protected constructor Create( leadBars : integer; systemName : string );
     // constructor to be used by derived classes in order to allow for symbol filtering in
     // addition to trade filtering
     protected constructor CreateWithSymbols( leadBars : integer; systemName : String; symbolFileName : String; separator : String );
     
  protected
     // trade filters. Return true if the order is to be entered, false
     // if the order is to be blocked
    function shortAtMarketFilter( Bar: integer; SignalName: string ) : boolean; override;
    function buyAtMarketFilter( bar : integer; SignalName : string ) : boolean; override;
    // symbol filter. Implements the symbol filtering using the list of symbols
    // in the file passed as parameter in the second constructor.
    // does not need to be changed or overriden - can be used as is.
    function symbolFilter( symbol : String ) : boolean; override;
  private
    function filter( bar : integer ) : boolean;
end;

constructor FilteredSystem.CreateWithSymbols( leadBars : integer; systemName : String; symbolFileName : String; separator : String );
begin
     Create( leadBars, systemName );
     _symbolsList := SymbolsList.Create( symbolFileName, '', separator );
     _hasSymbolsList := true;
end;

constructor FilteredSystem.Create( leadBars : integer; systemName : string );
begin
     inherited Create( leadBars, systemName );
     var bar : integer;

     _avgVol := SMASeries( #volume, 15 );
     _volatility := CreateSeries;

     // init volatility series
     for bar := 0 to barcount - 1 do
     begin
          SetSeriesValue( Bar, _volatility, HV( Bar, #Average, 20, 252 ) );
     end;
     _hasSymbolsList := false;
end;

// implements the filter for opening long positions
function FilteredSystem.buyAtMarketFilter( bar : integer; SignalName : String ) : boolean;
begin
     // calls the filter method
     Result := filter( bar );
end;

// implements the filter for opening short positions
function FilteredSystem.shortAtMarketFilter( bar : integer; SignalName : String ) : boolean;
begin
     Result := true;
//     Result := filter( bar );
end;

// implements the symbol filter - only symbols in the symbol list are allowed
function FilteredSystem.symbolFilter( symbol : String ) : boolean;
begin
     if( _hasSymbolsList ) then
     begin
         Result := _symbolsList.isInSymbolsList( symbol );
     end
     else
     begin
          Result := true;
     end;
end;

// this method only opens positions if the average volume for the last x bars was > 1000000 shares
// and if the average price for the previous bar was > $5/share.
function FilteredSystem.filter( bar : integer ) : boolean;
begin
     if( bar >= 1 ) then
     begin
       if( ( GetSeriesValue( bar - 1 , _avgVol ) >= 1000000 ) and ( PriceAverage( bar - 1  ) > 5 ) ) then // and ( GetSeriesValue( bar, _volatility ) > 125  ) ) then
       begin
         Result := true;
       end
       else
       begin
         Result := false;
       end;
     end;

end;
