(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This script contains a point by point description of the process of adapting a "regular" script for MultiSystem V1.1 .

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also uses these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//***************************************
// MultiSystem V1.1
//
// System Template - a point by point description of the process of
//        adapting a "regular" script for MultiSystem V1.1
//
// Adrian Michel 2003
//***************************************

//***************************************
// This portion (1-8) shows how to change a system to make it compatible with MultiSystem V1.0
//***************************************

// (1) This line includes the MultiSystem classes and makes them visible locally.
//	    It may not be necessary, or may even lead to multiple definitions in case
//	    there is a master system including all the separate components.
{$I 'System'}

// (2) Start class definition that will become the new system class.
//	    Note the syntax indicating that the new class is derived
//	    (or inherits from) the class System, defined in MultiSystem V1.0.
type System1 = class( System )

private
// (3) All the variables used by the system become data members of the new class.
//	    They can be declared private (preferred), which allows for better encapsulation.
//	    Also, they do not need to be prefixed by the var keyword.
constructor Create;

// (4) Declaration of the two main methods: the constructor Create, and runBar.
//	    Create is the class constructor, and RunBar is a method that overrides
//	    an abstract method with the same name in the class System. This allows
//	    for the polymorphic behavior, where the MultiSystem class can run any
//	    system class, as long as it is derived from System, the only class
//	    MultiSystem knows of.
protected procedure runBar( Bar : integer );override;

// (5) Optional - declaration of various filtering methods
//     The default behavior is 'on', so if these filters are not overriden,
//     they will not have any effect
//     If they return false, the buy/sell/short/cover order will not be entered, fact which can be
//     tested using the return value for these methods (in MultiScript, even atmarket and atclose
//     orders return a boolean value, to indicate whether the position was opened or not.
//     Here are only 2 of the many possible filters
function buyAtCloseFilter( Bar : integer; SignalName : string ) : boolean; override;
function buyAtLimitFilter( Bar : integer; LimitPrice : float; SignalName : string ) : boolean; override;

// (7) called with a symbol name - if Result is true, the system will be run
//     on this symbol, false otherwise. This can be used in conjuction with the
//     SymbolList class to create virtual sub-watchlists
function symbolFilter( symbol : String ) : boolean; override;

// (8) end of class definition
end;

// (9) Definition of the constructor Create. This is implemented as a regular
//	    procedure prefixed by the class name, and contains all the initialization
//	    code (all the code that is between the variable declaration and the main
//	    Bar loop, in a typical system).
constructor System1.Create;
begin
// (10) The constructor needs to call the base class (System) constructor with
//	     two parameters: number of lead bars and the system name.
	inherited Create( leadBars, systemName );
end;

// (11) runBar contains the main bar loop, where trades are generated.
//	    It does not need the for statement, as this method will be called
//	    from within MultiSystem for each bar.
procedure System1.runBar( Bar : integer );
begin
end;

// (12) implementation of the buy at close filter - see FilteredSystem V1.1 for a usage example
//      This implementation returns true
function System1.buyAtCloseFilter( Bar : integer; SignalName : string ) : boolean; override;
begin
     Result := true;
end;

// (13) implementation of the buy at limit filter - see FilteredSystem V1.1 for a usage example
//      This implementation returns true
function System1.buyAtLimitFilter( Bar : integer; LimitPrice : float; SignalName : string ) : boolean; override;
begin
     Result := true;
end;

// (14) implementation of the symbol filter - see FilteredSystem V1.0.2 for a usage example
//      This implementation returns true
function FilteredSystem.symbolFilter( symbol : String ) : boolean;
begin
     Result := true;
end;

//********************************************
// This portion shows how to write the multisystem part, where all the individual
// systems will be integrated and run as one. The systems and the master system
// will typically be implemented as different scripts
//********************************************

// (15) a variable of type MultiSystem is created
var ms : MultiSystem;

// (16) a MultiSystem object is created
ms := MultiSystem.Create;

// (17) variables of types of different systems are declared
var sys1, sys2, sys3 : System;

// (18) the system objects are created
sys1 := System1.Create;
sys2 := System2.Create;
sys3 := System3.Create;

// (19) the systems are made known to MultiSystem
ms.addSystem( sys1 );
ms.addSystem( sys2 );
ms.addSystem( sys3 );

// (20) MultiSystem is run by bar (there could be other ways,
//	such as running all the bars in one system, going to the next etc.
ms.runByBar;

// (21) free all the objects and resources used by the script.
ms.Free;
