(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This is the entry point script for the MultiSystem V1.1 framework.

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

This version adds some new capabilities and fixes a number of bugs found in previous versions. I strongly recommend that those who use any of the previous versions start using this version instead.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also uses these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//***********************************
// MultiSystem V1.1
//
// Definitions of System and MultiSystem classes
// main entry point in MultiSystem
//
// Adrian Michel 2003
//***********************************

{$I 'MultiSystem V1.1 FX'}
{$I 'MultiSystem V1.1 Positions'}

//***********************************
// System class extends Positions
//
// abstract class that makes the connection between MultiSystem and each concrete system that needs to be run
// it contains two 'abstract' methods init and runBar that need to be implemented by any class derived from System
// these methods are called polymorphically by MultiSystem in order to run the different systems
//***********************************
type
  System = class( Positions )
  private
    _leadBars : integer;
    _systemName : String;

  protected
  constructor Create( leadBars : integer; systemName : String );

  public
     //TODO: make this abstract when it becomes possible
  procedure init; virtual;
  procedure runBar( Bar : integer ); virtual;
  procedure run; virtual;

  protected
  procedure appendSystemNameToTradeName;
  procedure prependSystemNameToTradeName;

  public
  function getLeadBars : integer;
  function symbolFilter( symbol : String ) : boolean; virtual;

end;

function System.symbolFilter( symbol : String ) : boolean;
begin
    // by default don't filter
  Result := true;
end;

procedure System.appendSystemNameToTradeName;
begin
  appendStrToTradeName( _systemName );
end;

procedure System.prependSystemNameToTradeName;
begin
  prependStrToTradeName( _systemName );
end;

constructor System.create( leadBars : integer; systemName : String );
begin
  inherited Create;
  _leadBars := leadBars;
  _systemName := systemName;
  prependSystemNameToTradename;
end;


procedure System.init;
begin
  if assertOn then
    assert( false, 'call into System.init - this is "abstract"' );
end;

procedure System.runBar( Bar : integer );
begin
  if assertOn then
    assert( false, 'call into System.doBar - this is "abstract"' );
end;

procedure System.run;
begin
  if assertOn then
    assert( false, 'call into System.do - this is "abstract"' );
end;

function System.getLeadBars : integer;
begin
  Result := _leadBars;
end;

//***********************************
// MultiSystem class
//
// this implements the multi-system concept
// it contains a list of System derived objects and calls polymorphically different methods (such as runBar)
// thus running each system.
// individual systems are added using addSystem
// all systems are then run using either runByBar (runs one bar of each system at a time) or runBySystem (runs each system on all bars )
// for real time, runByBar is more appropriate, for simulation runBySystem can be used as well
//***********************************
type MultiSystem = class
  private _systems : TList;
//  private _leadBars : integer;

       //construction, destruction
  public
    constructor Create;
    procedure Free;

  public
    procedure addSystem( sys : System );
      //clears the systems list and frees the system objects
    procedure removeAllSystems;

    procedure runBySystem;
    procedure runByBar;
    procedure runByBarNoSymbolFilter;
    function barFilter( bar : integer ): boolean; virtual;
end;

constructor MultiSystem.Create;
begin
  _systems := TList.Create;
//  _leadBars := -1;
end;

procedure MultiSystem.Free;
begin
  removeAllSystems;
    // then free the TList
  _systems.Free;
end;

procedure MultiSystem.removeAllSystems;
begin
  var n : integer;
    // first free all the systems
  for n := _systems.count - 1 downto 0 do
  begin
    _systems.delete( n );
  end;
    // then free the TList
end;

procedure MultiSystem.addSystem( sys : System );
begin
  assert( sys.getLeadBars >= 0, 'Lead bars needs to be a positive number' );
{  if( _leadBars < 0 ) then
  begin
    _leadBars := sys.getLeadBars;
  end
  else
    _leadBars := min( _leadBars, sys.getLeadBars );
}
  _systems.addObject( 0, sys );
end;

procedure MultiSystem.runBySystem;
begin
  var count : integer;
  var sys : System;

  for count := 0 to _systems.count - 1 do
  begin
    sys := _systems.Object( count ) as System;
    sys.run;
  end;
end;

function MultiSystem.barFilter( bar : integer ): boolean;
begin
  Result := true;
end;

procedure MultiSystem.runByBarNoSymbolFilter;
begin
  var count : integer;
  var sys : System;
  var Bar : integer;

  for Bar := 0 to BarCount - 1 do
  begin
    begin
    for count := 0 to _systems.count - 1 do
    begin
      sys := _systems.Object( count ) as System;
       // TODO: don't do switch context for now, but be ready if it's needed for
       //  SetPositionSize and other currently global functions.
       //  sys.switchContext;
      if( Bar >= sys.getLeadBars ) then
        sys.runBar( Bar );
    end
    end;
  end;
end;


procedure MultiSystem.runByBar;
begin
  var count : integer;
  var sys : System;
  var Bar : integer;

  for Bar := 0 to BarCount - 1 do
  begin
//       Print( 'bar: ' + IntToStr( bar ) );
//    if( barFilter( bar ) ) then
//    begin
    for count := 0 to _systems.count - 1 do
    begin
//      Print( 'Sytem: ' + IntToStr( count + 1 ) );
      sys := _systems.Object( count ) as System;
      if( sys.symbolFilter( GetSymbol ) ) then
      begin
       // TODO: don't do switch context for now, but be ready if it's needed for
       //  SetPositionSize and other currently global functions.
       //  sys.switchContext;
        if( Bar >= sys.getLeadBars ) then
          sys.runBar( Bar );
      end;
//    end
    end;
  end;
end;
